/*********************************************/
/* Exemplo de uso de macros e funes para   */
/* tratamento de excees de ponto flutuante */
/*                                           */
/* Funes demonstradas:                     */
/*       * fegetexceptflag()                 */
/*       * feclearexcept()                   */
/*       * fetestexcept()                    */
/*       * fesetexceptflag()                 */
/*********************************************/

#include <stdio.h>
#include <float.h>
#include <fenv.h>

/***
 *
 * Suponha que a funo a seguir executa uma operao
 * de ponto flutuante que possa gerar uma exceo do
 * tipo representada pela flag FE_INVALID e que esta
 * funo deva manter intacto o status da varivel
 * que armazena todas as excees de ponto flutuante
 *
 ***/
double UmaFuncao(double x, double y)
{
   int        flag;
   fexcept_t  flagsGuardadas;

      /* Armazena todas as flags de excees de      */
      /* ponto flutuante na varivel fegetexceptflag */
   fegetexceptflag(&flagsGuardadas, FE_ALL_EXCEPT);

      /* Desliga a flag que supostamente interessa */
   feclearexcept(FE_INVALID);

      /* Executa a operao que pode */
      /* gerar a suposta exceo     */
   /* ... */

      /* Armazena na varivel flag a flag de interesse */
      /* (i.e., aquela que pode ter sido ligada) na    */
      /* operao representada acima por "..."         */
   flag = fetestexcept(FE_INVALID);

      /* Agora testa se ocorreu a provvel execeo */
   if (flag & FE_INVALID) {
         /* Faz o devido tratamento */
         /* da situao de exceo  */
      /* ... */
   }

      /* Antes de retornar, restaura todos */
      /* os sinalizadores de excees de   */
      /* ponto flutuante armazenados na    */
      /* varivel flagsGuardadas           */
   fesetexceptflag(&flagsGuardadas, FE_ALL_EXCEPT);

      /* Isto  apenas um programa educativo; por */
      /* isso,  retornado x + y. Na realidade,   */
      /* seria retornado o resultado da operao  */
      /* implementada pela funo.                */
   return x + y;
}

int main()
{
   double resultado;

      /* Executa operaes que podem gerar  */
      /* excees de ponto flutuante        */
   /* ... */

      /* Talvez algum sinalizador de exceo de */
      /* ponto flutuante esteja ligado neste    */
      /* ponto do programa. A chamada da funo */
      /* UmaFuncao() garante que o status dos   */
      /* sinalizadores ser preservado.         */

   resultado = UmaFuncao(2.4, -0.5);

   return 0;
}

/*

Resultado do programa no Windows XP:

Este programa no tem sada, est OK.

*/
